from cdlake._internal import CdlFS
import pyarrow as pa
import torch


class CdlTorchDataset(torch.utils.data.Dataset[bytes]):
    def __init__(
        self,
        batch: pa.RecordBatch,
        batch_size: int,
        fs: CdlFS,
    ) -> None:
        super().__init__()
        self._batch = batch
        self._batch_size = batch_size
        self._fs = fs

    def __getitem__(
        self,
        index: int,
    ) -> bytes:
        return self.__getitems__([index])[0]

    def __getitems__(
        self,
        indices: list[int],
    ) -> list[bytes]:
        print(self._batch.take(indices))
        return self._fs.read_files(self._batch.take(indices))

    def __len__(self) -> int:
        return len(self._batch)



from enum import Enum

import pyarrow as pa


class Compression(Enum):
    BROTLI = 0
    GZIP = 1
    LZO = 2
    LZ4 = 3
    LZ4_RAW = 4
    SNAPPY = 5
    UNCOMPRESSED = 6
    ZSTD = 7


class Url:
    def __init__(self, url: str, /) -> None: ...


class DatasetCatalog:
    compression: Compression
    compression_level: int | None = None
    max_buffer_size: int
    max_chunk_size: int
    max_write_threads: int
    s3_access_key: str | None = None
    s3_endpoint: Url
    s3_region: str
    s3_secret_key: str | None = None


class CdlFS:
    def copy_to(self, dst: str, /) -> None: ...

    def read_dir(self, path: str = '/', /) -> pa.RecordBatch: ...

    def read_dir_all(self, /) -> pa.RecordBatch: ...

    def read_files(self, files: pa.RecordBatch, /) -> list[bytes]: ...

    def sql(self, sql: str, /) -> pa.RecordBatch: ...


class Cdl:
    def __init__(
        self, /,
        catalog: DatasetCatalog | None = None,
    ) -> None: ...

    def open(self, url: str, /) -> CdlFS: ...



import pyarrow as pa

from ._internal import CdlFS as _CdlFSImpl


class CdlFS:
    def __init__(self, impl: _CdlFSImpl) -> None:
        self._impl = impl

    def copy_to(self, dst: str) -> None:
        return self._impl.copy_to(dst)

    def read_dir(
        self,
        path: str,
    ) -> pa.RecordBatch:
        return self._impl.read_dir(path)

    def read_dir_all(self) -> pa.RecordBatch:
        return self._impl.read_dir_all()

    def sql(
        self,
        sql: str,
    ) -> pa.RecordBatch:
        return self._impl.sql(sql)

    def to_torch_dataset(
        self,
        batch_size: int = 1,
    ):
        from cdlake.dataset import CdlTorchDataset
        return CdlTorchDataset(
            batch=self.read_dir_all(),
            batch_size=batch_size,
            fs=self._impl,
        )



from ._internal import (Cdl as _CdlImpl, DatasetCatalog)
from .filesystem import CdlFS


class Cdl:
    def __init__(
        self,
        catalog: DatasetCatalog | None = None,
    ) -> None:
        self._impl = _CdlImpl(catalog)

    def open(self, url: str) -> CdlFS:
        return CdlFS(self._impl.open(url))
